//
// file      : gfx1_ser_dr.c
// author    : Ian Forse
// date      : 22-MAY-2004
// purpose   : SP 5-GFX1 low level drivers
// includes  : gfx1_ser_dr.h
// notes     : 
// compiler  : Hi tech Compiler V8.00PL3
//
// Copyright (C) 2004 Ian Forse
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
// USA


#include "gfx1_ser_dr.h"

// Sunplus SPLC501C Constants
#define gfx1_Page_base 0xB0
#define gfx1_Col_base_l 0x00
#define gfx1_Col_base_h 0x10

// Port definitions. Change as required.
#define gfx1_A0P RB3
#define gfx1_Reset RE2
#define gfx1_SI RD7
#define gfx1_SCL RD6
#define gfx1_A0P_direction TRISB3
#define gfx1_Reset_direction TRISE2
#define gfx1_SI_direction TRISD7
#define gfx1_SCL_direction TRISD6


#define high (1)
#define low (0)
#define input (1)
#define output (0)


// ----------------------------------------------------------------------------
// Shift byte x out to the display as serial.  
// Note range is from 0 i.e. 8 bits is  0 to 7.
// ----------------------------------------------------------------------------

void gfx1_ser_output( unsigned char x )
{
	signed char i;
		for(i=7; i>=0; i--)
		{
			if ((x>>i)&0x01)
			{
				gfx1_SI = high;
			}
			else
			{
				gfx1_SI = low;
			}
			gfx1_SCL = low;
			gfx1_SCL = high;
		}
}


// Write commands i.e. page and column addresses
void gfx1_write_command (unsigned char data1 )
{
  	gfx1_A0P = low;
  	gfx1_ser_output( data1 );
}


// Write display data
void gfx1_write_data (unsigned char data2 )
{
  	gfx1_A0P = high;
  	gfx1_ser_output( data2 );
}


// Write display data to a position. Parameters are page, column and data.
void gfx1_write_pos (unsigned char gfx1_page, unsigned char gfx1_col, unsigned char gfx1_data )
{
	gfx1_write_command (gfx1_Page_base + ( gfx1_page ));
  	gfx1_write_command (gfx1_Col_base_h + ( ( gfx1_col & 0xF0 ) >> 4 ));
  	gfx1_write_command (gfx1_Col_base_l + ( gfx1_col & 0x0F ));
  	gfx1_write_data (gfx1_data);
}


// Write to the next location.  This is only used after a write to a known location
void gfx1_write_next ( unsigned char data3 )
{
	gfx1_write_data ( data3 );
}


// Clear one page of display (pages 0 to 7)
void gfx1_clear_page (unsigned char page_to_clear)
{
	unsigned char i;
	gfx1_write_pos ( page_to_clear , 0x00 , 0x00 );
	for (i=0; i<128; i++)
	{
  		gfx1_write_next ( 0x00 );
  	}
}


// Clear all pages of display (pages 0 to 7)
void gfx1_clear_all ( void )
{
	unsigned char i;
	for (i=0; i<8; i++)
	{
  		gfx1_clear_page ( i );
  	}
}


// Initialisation of the 16F877 I/O and SPLC501C
void gfx1_start_up (void)
{
	gfx1_A0P = low;
	gfx1_Reset = high;
	gfx1_Reset_direction = output;
	gfx1_A0P_direction = output;
	gfx1_SI_direction = output;
	gfx1_SCL_direction = output;

	// Reset
	gfx1_Reset = low;	// Minimum required by SPLC501C is 0.5us
	#asm			// Using a PIC 16F877 @ 4Mhz
	nop			// this instruction takes 1us
	#endasm
	gfx1_Reset = high;
	
	// Initialisation.  6.xx refers to the section of the Sunplus SPLC501C Datasheet
	gfx1_write_command ( 0xA2 );		// LCD Bias Set - 6.11
	gfx1_write_command ( 0xA0 );		// Normal Display Mode - 6.10
	gfx1_write_command ( 0xC8 );		// Common Output Mode - 6.15
	gfx1_write_command ( 0x23 );		// V5 Voltage Reg Internal Resistor - 6.17
	gfx1_write_command ( 0x81 );		// Electronic Volume Double Command - 6.18
	gfx1_write_command (EEPROM_READ(0x00));	// This is the internal PIC EEPROM @ address
						// 00. Default data is 0x1F.  In main code
						// this is adjustable from 0x00 to 0x3F and
						// then stored back into the EEPROM.
	gfx1_write_command ( 0x2B );		// Power Control Circuit - 6.16
	gfx1_write_command ( 0x40 );		// Display Start Line Set - 6.2
	gfx1_write_command ( gfx1_Page_base );	// Page Address set - 6.3
	gfx1_write_command ( gfx1_Col_base_h );	// Column address set upper bit - 6.4
	gfx1_write_command ( gfx1_Col_base_l );	// Column address set upper bit - 6.4
	gfx1_write_command ( 0xAF );		// DISPLAY ON - 6.1
}


// Set the start position.
void gfx1_start_posn  (unsigned char start_col, unsigned char start_page)
{
	gfx1_write_command (gfx1_Page_base + ( start_page ));
  	gfx1_write_command (gfx1_Col_base_h + ( ( start_col & 0xF0 ) >> 4 ));
  	gfx1_write_command (gfx1_Col_base_l + ( start_col & 0x0F ));
}




